(ns cavm.h2-query
  (:require [clojure.core.match :refer [match]])
  (:gen-class))

; optimization cases
;  {:select foo
;   :from bar
;   :where [:in column literal]}
;
;  maps to
;
;  {:select foo
;   :from bar
;   :join [[:#sqlcall :table :val literal F] [:= column F.val]]}
;
;  {:select foo
;   :from bar
;   :where [:in column {:select :column2 :from :blah}])
;
;   maps to
;
;   {:select foo
;    :from bar
;    :join [[{select :column2 :from :blah} F] [:= F.column2 :column]]}
;
;   {:select foo
;    :from bar
;    :where [:and [:in column literal] [:= :column2 3]]}
;
;   maps to
;
;   {:select foo
;    :from bar
;    :join [[:#sqlcall :table :val literal F] [:= column F.val]]
;    :where [:= :column2 3]}
;
;   {:select foo
;    :from bar
;    :where [:or [:in column literal] [:= :column2 3]]}
;
;   If we inner-join we lose the columns that would match column2==3. We could
;   perform a union, perhaps.
;
;   {:union
;     {:select foo
;      :from bar
;      :join [[:#sqlcall :table :val literal F] [:= column F.val]]}
;     {:select foo
;      :from bar
;      :where [:= :column2 3]}}
;
;   {:select foo
;    :from bar
;    :where [:or [:and [:in column literal] [:= :column2 3]] [:= :column3 "blah"]]}
;
;   This can be split using the above two patterns. Does this work in general?
;   What about :not, or multiple :in clauses?
;
;   [:not [:in column literal]]
;
;   There's no way to optimize this with indexes, so don't bother. Unclear how
;   to symplify to establish if a :in clause is positive or negative in nature.
;
;   What about mutiple :in clauses?
;
;   {:select foo
;    :from bar
;    :where [:and [:in column literal] [:in column2 literal2]]}
;
;   {:select foo
;    :from bar
;    :join [[:#sqlcall :table :val literal F] [:= column F.val]
;          [:#sqlcall :table :val literal2 F2] [:= column2 F.val]]}


;
; :where clauses with :and :or :not
; One wants to break down the boolean expression such that the :in clauses
; are separable.
; [:and [:or [:in] [:= A]] [:= B]]
; The :and doesn't decompose into multiple selects, or a join.
; [:or [:and [:in := A] [:and [:= A] [:= B]]]]
; This decomposes into a union over a join and a select.

(defn distribute-not [exp]
  (match exp
         [:not [:not x]] (distribute-not x)
         [:not [:and & params]] (vec (concat [:or] (for [p params]
                                                     (distribute-not [:not p]))))
         [:not [:or & params]] (vec (concat [:and] (for [p params]
                                                     (distribute-not [:not p]))))
         :else exp))

; (for [i [1 2 3]] [:not i])
; (distribute-not [:not [:and 'A 'B]])
; (distribute-not [:not [:or 'A 'B 'C]])
; (distribute-not [:not [:not 'A]])
; (distribute-not [:not [:not [:not 'A]]])
; (distribute-not [:not [:not [:not [:not 'A]]]])
; (distribute-not [:not [:and [:or 'A 'B 'C] [:not 'D]]])

(defn opt-where-in [exp]
  (match exp
         {:select x
          :where [:in column value]} [x column value]
         :else exp))

;  (opt-where-in {:foo :bar})
;  (opt-where-in {:select [:dataset]})
